'''
  $Id: helpers.py 192 2021-03-11 15:19:05Z cl_yubikey_2018a $
  
  Copyleft ITEG IT-Engineers GmbH 2022
'''

#import base64
#import configparser
#import glob
import os.path
#import re
#import socket
import sys
#import zipfile

from urllib.parse import urlparse
#from xml.dom.minidom import parse
import xml.dom.minidom

#from clazzes.pba import helpers
from clazzes.pba import pbalogging

log = pbalogging.getLogger()


class PBAConfiguration:
    '''
      holds eventual parsed configuration and deb/rpm/jar/pom/... metadata
    '''

        # static holder for first instance
    staticPBAConfiguration = None

    def __init__(self): 
        PBAConfiguration.staticPBAConfiguration = self
        

class PomXml:
    ''' partially based on jartools` dto.py '''
    
    def __init__(self, location:str = "", parentPomXml = None, pbaConfiguration:PBAConfiguration = None):
        self._clearCoordinates()
        self.location = location
        self.path = None
        self.pbaConfiguration = pbaConfiguration
        self.parentPomXml = parentPomXml

    def _clearCoordinates(self):
        # properties to get from pom xml or pom properties
        self.groupId = None
        self.artifactId = None
        self.version = None
        self.name = ""
        # children
        self.modulesPomXmls = None
        # publishing parts
        self.releaseRepositoryUrl = None
        self.snapshotRepositoryUrl = None
        self.javaDocSyncTarget = None
        # status helper
        self.haveCoordinates = False

    def parsePomXml(self, path:str = ""):
        self.path = path
        log.debug("PomXml.parsePomXml(): Reading %s ..." %
                  self.path)
        domTree = xml.dom.minidom.parse(self.path)
        xmlCollection = domTree.documentElement
        #for xmlObject in xmlCollection.getElementsByTagName("object"):
        self._clearCoordinates()
        parentNode = None
        parentGroupIdNode = None
        parentArtifactIdNode = None
        parentVersionNode = None
        parentNameNode = None
        groupIdNode = None
        artifactIdNode = None
        versionNode = None
        nameNode = None
        modulesNode = None
        modulesNodes = []
        distributionManagementNode = None
        for pomNode in xmlCollection.childNodes:
            if pomNode.nodeType != pomNode.ELEMENT_NODE:
                continue
            if pomNode.nodeName == "parent":
                parentNode = pomNode
                continue
            if pomNode.nodeName == "groupId":
                groupIdNode = pomNode
                continue
            if pomNode.nodeName == "artifactId":
                artifactIdNode = pomNode
                continue
            if pomNode.nodeName == "version":
                versionNode = pomNode
                continue
            if pomNode.nodeName == "name":
                nameNode = pomNode
                continue
            if pomNode.nodeName == "modules":
                modulesNode = pomNode
                continue
            if pomNode.nodeName == "distributionManagement":
                #log.debug("PomXml.parsePomXml(): Found distributionManagement")
                distributionManagementNode = pomNode
                continue
        if parentNode != None:
            for pomNode in parentNode.childNodes:
                if pomNode.nodeName == "groupId":
                    parentGroupIdNode = pomNode
                    continue
                if pomNode.nodeName == "artifactId":
                    parentArtifactIdNode = pomNode
                    continue
                if pomNode.nodeName == "version":
                    parentVersionNode = pomNode
                    continue
                if pomNode.nodeName == "name":
                    parentNameNode = pomNode
                    continue
        if modulesNode != None:
            for pomNode in modulesNode.childNodes:
                if pomNode.nodeName == "module":
                    modulesNodes.append(pomNode)
                    continue
        dmRepositoryNode = None
        dmSnapshotRepositoryNode = None
        if distributionManagementNode != None:
            for pomNode in distributionManagementNode.childNodes:
                if pomNode.nodeName == "repository":
                    #log.debug("PomXml.parsePomXml(): Found dmRepositoryNode")
                    dmRepositoryNode = pomNode
                    continue
                if pomNode.nodeName == "snapshotRepository":
                    #log.debug("PomXml.parsePomXml(): Found dmSnapshotRepositoryNode")
                    dmSnapshotRepositoryNode = pomNode
                    continue
        dmRepositoryUrlNode = None
        if dmRepositoryNode != None:
            for pomNode in dmRepositoryNode.childNodes:
                if pomNode.nodeName == "url":
                    #log.debug("PomXml.parsePomXml(): Found dmRepositoryUrlNode")
                    dmRepositoryUrlNode = pomNode
                    continue
        dmSnapshotRepositoryUrlNode = None
        if dmSnapshotRepositoryNode != None:
            for pomNode in dmSnapshotRepositoryNode.childNodes:
                if pomNode.nodeName == "url":
                    #log.debug("PomXml.parsePomXml(): Found dmSnapshotRepositoryUrlNode")
                    dmSnapshotRepositoryUrlNode = pomNode
                    continue
        # parse xml
        # my coordinates
        if groupIdNode != None:
            self.groupId = groupIdNode.firstChild.data
        elif parentGroupIdNode != None:
            self.groupId = parentGroupIdNode.firstChild.data
        if artifactIdNode != None:
            self.artifactId = artifactIdNode.firstChild.data
        if versionNode != None:
            self.version = versionNode.firstChild.data
        elif parentVersionNode != None:
            self.version = parentVersionNode.firstChild.data
        elif self.parentPomXml != None:
            self.version = self.parentPomXml.version 
        if nameNode != None:
            self.name = nameNode.firstChild.data
        # parent coordinates
        if self.parentPomXml == None: # we might have that injected that in the c'tor in the future
            parentLocation = "Parent of %s" % self.location
            self.parentPomXml = PomXml(location=parentLocation, pbaConfiguration=self.pbaConfiguration)
            nrParentParts = 0
            if parentGroupIdNode != None:
                self.parentPomXml.groupId = parentGroupIdNode.firstChild.data
                nrParentParts += 1
            if artifactIdNode != None:
                self.parentPomXml.artifactId = artifactIdNode.firstChild.data
                nrParentParts += 1
            if parentVersionNode != None:
                self.parentPomXml.version = parentVersionNode.firstChild.data
                nrParentParts += 1
            if parentNameNode != None:
                self.parentPomXml.name = parentNameNode.firstChild.data
            if parentGroupIdNode != None:
                self.parentPomXml.groupId = parentGroupIdNode.firstChild.data
            if parentArtifactIdNode != None:
                self.parentPomXml.artifactId = parentArtifactIdNode.firstChild.data
            if nrParentParts < 3:
                # clear incomplete parent object
                self.parentPomXml = None
            else:
                self.parentPomXml.haveCoordinates = True
        # repos
        if dmRepositoryUrlNode != None:
            self.releaseRepositoryUrl = dmRepositoryUrlNode.firstChild.data
            #log.debug("Parsed releaseRepositoryUrl from dmRepositoryUrlNode: %s" % self.releaseRepositoryUrl)
        elif self.parentPomXml != None:
            self.releaseRepositoryUrl = self.parentPomXml.releaseRepositoryUrl
            #log.debug("Picked up parentPomXml.releaseRepositoryUrl: %s" % self.releaseRepositoryUrl)
        else:
            log.debug("No dmRepositoryUrlNode and no parentPomXml for %s" % self.path)
        if dmSnapshotRepositoryUrlNode != None:
            self.snapshotRepositoryUrl = dmSnapshotRepositoryUrlNode.firstChild.data
            #log.debug("Parsed snapshotRepositoryUrl from dmSnapshotRepositoryUrlNode: %s" % self.snapshotRepositoryUrl)
        elif self.parentPomXml != None:
            self.snapshotRepositoryUrl = self.parentPomXml.snapshotRepositoryUrl
            #log.debug("Picked up parentPomXml.snapshotRepositoryUrl: %s" % self.snapshotRepositoryUrl)
        else:
            log.debug("No dmSnapshotRepositoryUrlNode and no parentPomXml for %s" % self.path)
        # future: more?
        # modules
        #if modulesNode != None
        modulesNode = None
        if len(modulesNodes):
            self.modulesPomXmls = []
            pomDir = os.path.dirname(self.path)
            for modulesNode in modulesNodes:
                moduleRelPath = modulesNode.firstChild.data 
                modulePomXmlPath = pomDir + "/" + moduleRelPath + "/pom.xml"
                log.debug("Computed modulePomXmlPath to %s" % modulePomXmlPath)
                moduleLocation = "Child %s of %s" % (moduleRelPath, self.location)
                #log.debug("Creating modulePomXml for module path %s" % moduleRelPath))
                modulePomXml = PomXml(location=moduleLocation, parentPomXml=self, pbaConfiguration=self.pbaConfiguration)
                modulePomXml.parsePomXml(path=modulePomXmlPath)
                self.modulesPomXmls.append(modulePomXml)
        # evaluate
        if self.groupId == None:
            log.error("PomXml.parsePomXml(): %s did not provide a groupId" % self.path)
            sys.exit(1)
        if self.artifactId == None:
            log.error("PomXml.parsePomXml(): %s did not provide a artifactId" % self.path)
            sys.exit(1)
        if self.version == None:
            log.error("PomXml.parsePomXml(): %s did not provide a version" % self.path)
            sys.exit(1)
        if self.name == None:
            log.info("PomXml.parsePomXml(): %s did not provide a name" % self.path)
        log.debug("PomXml.parsePomXml(): %s did provide groupId '%s', artifactId '%s', version '%s', name '%s'" % (
            self.path, self.groupId, self.artifactId, self.version, self.name))
        self.haveCoordinates = True
        # javaDocSyncTarget URLs
        self.javaDocSyncTarget = None
        if self.version.endswith("-SNAPSHOT"):
            if self.snapshotRepositoryUrl != None:
                self.javaDocSyncTarget = PomJavaDocSyncTarget(pomXml = self, repoUrl = self.snapshotRepositoryUrl)
                self.javaDocSyncTarget.parseRepoUrl()
        else:
            if self.releaseRepositoryUrl != None:
                self.javaDocSyncTarget = PomJavaDocSyncTarget(pomXml = self, repoUrl = self.releaseRepositoryUrl)
                self.javaDocSyncTarget.parseRepoUrl()

    def describeCoordinates(self, descFormat:str=""):
        if descFormat == "text":
            return self._describeCoordinatesText()
        if descFormat == "txt":
            return self._describeCoordinatesTxt()
        if descFormat == "csv":
            return self._describeCoordinatesCsv()
        #if descFormat == "pom.xml":
        #    return self._describeCoordinatesPomXml()
        log.error("PomXml.printCoordinates(): Unsupported format '%s'" % descFormat)
        sys.exit(1)
        
    def _describeCoordinatesText(self):
        if not self.haveCoordinates:
            return "%s: -\n" % self.location
        desc = "%s:\n  groupId:    %s\n  artifactId: %s\n  version:    %s\n  repos:      %s and %s\n" % (
            self.location, self.groupId, self.artifactId, self.version, self.releaseRepositoryUrl, self.snapshotRepositoryUrl)
        if self.name == None:
            return desc
        if len(self.name) == 0:
            return desc
        desc += "  name:       %s\n" % self.name
        return desc

    def _describeCoordinatesTxt(self):
        if not self.haveCoordinates:
            return "%s: -\n" % self.location
        desc = "%s: groupId %s, artifactId %s, version %s\n" % (
            self.location, self.groupId, self.artifactId, self.version)
        if self.name == None:
            return desc
        if len(self.name) == 0:
            return desc
        desc = "%s: groupId %s, artifactId %s, version %s, name %s\n" % (
            self.location, self.groupId, self.artifactId, self.version, self.name)
        return desc

    def _describeCoordinatesCsv(self):
        if not self.haveCoordinates:
            return "%,,,\n"
        if self.name == None:
            return "%s,%s,%s,\n" % (
                self.groupId, self.artifactId, self.version)
        return "%s,%s,%s,%s\n" % (
            self.groupId, self.artifactId, self.version, self.name)


class PomJavaDocSyncTarget:
    def __init__(self, pomXml:PomXml = None, repoUrl:str = None):
        self.pomXml = pomXml
        self.repoUrl = repoUrl
        self.repoProtocol = None
        self.repoUser = None
        self.repoServer = None
        #self.repoPort = -1
        self.repoDocRoot = None
        self.repoJavaDocDir = None
        self.syncSource = os.path.dirname(pomXml.path) + "/target/site/apidocs"
        self.syncTarget = None
    
    def parseRepoUrl(self):
        parsedUrl = urlparse(self.repoUrl)
        self.repoProtocol = parsedUrl.scheme
        self.repoUser = parsedUrl.username # we might need to make this configurable
        self.repoServer = parsedUrl.hostname
        #self.repoPort = parsedUrl.port
        self.repoDocRoot = parsedUrl.path
        self.repoJavaDocDir = "%s/javadoc/%s/%s" % (
            self.repoDocRoot, self.pomXml.groupId, self.pomXml.artifactId)
        self.syncTarget = "%s:%s" % (self.repoServer, self.repoJavaDocDir) 
    
    def describe(self):
        desc = "PomJavaDocSyncTarget:\n"
        desc += "  Protocol: %s\n" % self.repoProtocol
        desc += "  User:     %s\n" % self.repoUser 
        desc += "  Server:   %s\n" % self.repoServer 
        #desc += "  Port:     %d\n" % self.port 
        desc += "  DocRoot:  %s\n" % self.repoDocRoot 
        desc += "  JavaDoc:  %s\n" % self.repoJavaDocDir
        return desc
