'''
  $Id: helpers.py 192 2021-03-11 15:19:05Z cl_yubikey_2018a $
  
  Copyleft ITEG IT-Engineers GmbH 2022
'''

import fileinput
#import ipaddress
#import json
#import os
import re
#import socket
import subprocess
import sys

from clazzes.pba import pbalogging

log = pbalogging.getLogger()


def formatFileSize(size:int = -1):
    if size == None:
        return "- (None)"
    if size < 0:
        return "- (< 0)"
    if size < 1024:
        return "%d B" % size
    if size < 1048576:
        return "%d KB" % (size/1024)
    if size < 1073741824:
        return "%d MB" % (size/1048576)
    if size < 1099511627776:
        return "%d GB" % (size/1073741824)
    return "%1.2f TB" % (size/1099511627776)


def parseFileSize(sizeString:str = ""):
    if sizeString == None:
        return -1
    if len(sizeString) == 0:
        return -2
    reInt = re.compile("^\d+$")
    #matchInt = reInt.search(sizeString)
    if reInt.fullmatch(sizeString):
        return int(sizeString)
        
    reIntUnit = re.compile("^(\d+)([mgtMGT])$")
    matchIntUnit = reIntUnit.fullmatch(sizeString)
    if matchIntUnit == None:
        return -3
    sizeInt = int(matchIntUnit.group(1))
    sizeBytes = -4
    if matchIntUnit.group(2) in "mM":
        sizeBytes = sizeInt * 1048576
    elif matchIntUnit.group(2) in "gG":
        sizeBytes = sizeInt * 1073741824
    elif matchIntUnit.group(2) in "tT":
        sizeBytes = sizeInt * 1099511627776
    #log.debug("parseFileSize(): Computed '%s' to %d" % (
    #    sizeString, sizeBytes))
    return sizeBytes


def readConf(confFileName:str = ""):
    if len(confFileName) == 0:
        raise Exception("ERROR: readConf() called with empty confFileName")
    conf = {}
    lineNr = 0
    for line in fileinput.input(files=(confFileName)):
        lineNr += 1
        if line[-1] == '\n':
            line = line[:-1]
        #log.debug("readConf(%s:%d): line='%s'" % (confFileName, lineNr, line))
        if len(line) == 0:
            #log.debug("readConf(%s:%d): Ignoring emtpy line" % (confFileName, lineNr))
            continue
        if line[0] in "#;":
            #log.debug("readConf(%s:%d): Comment line" % (confFileName, lineNr))
            continue
        lineParts = line.split('=')
        if len(lineParts) < 1:
            log.warning("readConf(%s:%d): Ignoring invalid line wihtout '=': '%s'" % (
                confFileName, lineNr, line))
            continue
        if len(lineParts[0]) == 0:
            log.warning("readConf(%s:%d): Ignoring invalid line without option name: '%s'" % (
                confFileName, lineNr, line))
            continue
        name = lineParts[0].strip()
        if len(name) == 0:
            log.warning("readConf(%s:%d): Ignoring invalid line with empty option name: '%s'" % (
                confFileName, lineNr, line))
            continue
        #log.debug("readConf(%s:%d): Trimmed name: '%s'" % (confFileName, lineNr, name))
        value = "=".join(lineParts[1:]).strip()
        # empty value is allowed
        #log.debug("readConf(%s:%d): Trimmed value: '%s'" % (confFileName, lineNr, value))
        if len(value) > 1 and value[0] == '"' and value[-1] == '"':
            value = value[1:-1]
        for key in conf.keys():
            value = value.replace("$%s" % key, conf[key]).replace("${%s}" % key, conf[key])
        #log.debug("readConf(%s:%d): $-replaced value: '%s'" % (confFileName, lineNr, value))
        conf[name] = value
    
    return conf


def backcheckOnProblem(rc:int=-1, cmdParts:[]=[], problem:str=""):
    if len(cmdParts) > 0:
        print("ATTENTION: The former command,\n  %s\nreturned the bad exit code %d." % (
            " ".join(cmdParts), rc), file=sys.stderr, flush=True)
    else:
        print("ATTENTION: The former command returned the bad exit code %d." %
              rc, file=sys.stderr, flush=True)

    if len(problem) > 0:
        print("PROBLEM description: %s" %
              problem, file=sys.stderr, flush=True)

    choice = input("Please try to correct the problem and re-execute the command manually in another terminal.\n"
                   "If your are You sure it is ok to continue, enter 'yes' ... ")
    if choice != "yes":
        log.error("You chose to ABORT halfway because of the problem mentioned above.")
        sys.exit(1)

    log.warning("Continuing on user's request")    
    
    
def performDebugLogged(cmdParts:list=[]):
    if len(cmdParts) == 0:
        raise Exception("ERROR: performDebugLogged() called with empty cmdParts")
    log.debug("Running\n  %s" % " ".join(cmdParts))
    cp = subprocess.run(cmdParts)
    rc = cp.returncode
    if rc != 0:
        log.debug("Got bad exitcode %d from %s" % (
            rc,  " ".join(cmdParts)))
    else:
        log.debug("Returncode was: %d" % rc)
    return rc

def performPipeDebugLogged(cmdParts:list=[], input="", text:bool=True):
    if len(cmdParts) == 0:
        raise Exception("ERROR: performDebugLogged() called with empty cmdParts")
    log.debug("Running\n  %s" % " ".join(cmdParts))
    cp = subprocess.run(cmdParts, stdout=subprocess.PIPE, stderr=subprocess.PIPE, input=input, text=text)
    rc = cp.returncode
    if rc != 0:
        log.debug("Got bad exitcode %d from %s" % (
            rc,  " ".join(cmdParts)))
    else:
        log.debug("Returncode was: %d" % rc)
    return rc, cp.stdout, cp.stderr
